%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% LOOKING AT NCL FILES IN Matlab %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Rosie Fisher, 2014 CLM Tutorial
% Derived from code written also by Sanjiv Kumar and Kyla Dahlin


% WAYS To RUN MATLAB		     
% 1) On yellowstone you can either run matlab by typing
% >matlab
% (but only do this for very simple tasks)

% 2) To run in an interactive window (which is a more polite a socially concious thing to do)
% Open an interactive job in geyser as we did with NCL
% >bsub -Ip -q geyser -W 2:00 -n 1 -P <project_code> xterm
% then in that interactive window type 
% module load matlab
% >matlab

% NB>. At NCAR, we only have a limited number of matlab licences. Thus,
% matlab might not load if too many people try simultaneously. Don't worry,
% if you are interesting in matlab you can look at this at any point!

% 3) To run matlab -without- the graphical user interface, use
%> matlab -nodesktop
% This is much quicker, and you can edit your code in a different window using either nedit (quicker), gedit(prettier) or emacs (more portable)
% n.b. To use the nedit or gedit programs (simple text editors, you must login in yellowstone using the -XY 
% command. i.e. 
% ssh -YX user***@yellowstone.ucar.edu

% STEPPING THROUGH THE CODE ONE LINE AT A TIME
% For this tutorial, you might want to run the code one line at a time. To do this, you should use the GUI via method 2				  
% -> Open this matlab script in the GUI window (using 'open'). 
% If you press the big green arrow, the code will run all the way through
% To stop it running all the way through, you need to make a 'breakpoint' which tells the code to stop when it gets to that line
% -> Make a breakpoint by scrolling down to the first active line (opengl neverselect) and clicking your mouse on the '-' sign next to the line number
% The '-' should turn into a green circle. 
% -> Now press the 'run' button (if it asks, click 'change folder')
% A small green arrow will appear to the right of the green circle. 
% This is where you now are in the code
% Step through each line by clicking on the 'step' button above. 
% If you are stepping through a loop, you can click on a line after the
% bottom of the loop and use 'run to cursor' to get there faster. 

% force matlab to write graphics correctly.... 
opengl neverselect
					   
% If we re-run this script, we need to close all the existing figures and void the existing variables 
close all
clear all

%%%%%%%%%%%%%%% READ IN AND LOOK AT A SINGLE .nc file %%%%%%%%%%%%%%%%

% Tell the script where your model output files are to be found...
dir_clm = '/glade/scratch/dll/archive/CLM2014Tutorial_5yr1850test/lnd/hist/'

% Define the file you want to look at : you may need to change this:
filename = strcat(dir_clm,'/CLM2014Tutorial_5yr1850test.clm2.h0.0001-01.nc')

% Open this netcdf file (which you do not want to write to...), and give it an id number
fileid = netcdf.open(filename,'NC_NOWRITE')

% Choose a particular variable to plot from this netcdf file, and give it an ID number
varid = netcdf.inqVarID(fileid,'TSA')

% Read in the data from this file and variable. It's data type is 'double precision' so we need to say that here...
TSA = squeeze(netcdf.getVar(fileid,varid,'double'));
size(TSA); 

% This data needs to be 'squeezed' as it has a ghost dimension.
TSA = squeeze(TSA);

% Open a plotting window
figure

% Plot the air temperature martix
pcolor(TSA)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Look at the plot in the figure window.  Does it look OK?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%What do the colours mean?
% Add in the legend
colorbar

% All of the ocean is a huge number?
% Remove all of the very big numbers and turn them into 'not a number' (nan) which is plotted in white
TSA(find(TSA>10^34)) = nan;

% Try again
pcolor(TSA)  

% Get rid of all the black grid lines
shading flat

% The world seems to be flipped on its side. Why is that?  
% Invert the matrix (flip on both dimensions) so it's the right way around. 
TSA = TSA'; 

% Try again
pcolor(TSA)
shading flat;colorbar

% It's in Kelvin. No-one really thinks in Kelvin...
% Convert from Kelvin to degrees C. 
TSA = TSA - 273.15

% Try again
pcolor(TSA)
shading flat;colorbar

%Why are the axes lables not the same as the latitude and longitude?
% -- This is because Matlab doesn't yet know the latitude and longitude, it just thinks that we have a standard 2x2 grid
% To fix this, we need to read in the latitude and longitudes associated
% with the grid. 
varid = netcdf.inqVarID(fileid,'lon');                                                                                                                         
lon = squeeze(netcdf.getVar(fileid,varid,'double'));                                                                                                                
                                                                                                                                                                       
varid = netcdf.inqVarID(fileid,'lat');                                                                                                                  
lat = squeeze(netcdf.getVar(fileid,varid,'double'));                                                                                                                

% If a variable has 3 dimensions, but one of them has a length of 1
% then we used this 'squeeze' command to collapse it down to two dimensions
lat = squeeze(lat);                                                                                                                                                    
lon = squeeze(lon); 

% We need to make arrays for latitude and longitude the same size as the
% TSA grid. 

lats = repmat(lat,1,144);
lons = repmat(lon,1,96)';

% Try again
pcolor(lons,lats,TSA) 
shading flat; colorbar

% Add a title
title('Air temperature ^oC');

% Give this figure a name, to avoid confusion
set(gcf,'name','Temperatures inC January 01')


% Print out this file the easy way
saveas(gcf,'Air_temperatures.jpeg')

% Print out this figure the high resolution way
print -djpeg -r500 'Air_temp_highres.pdf'

%%%%%%%%% NOW DO THE SAME THING IN A LOOP TO READ IN THE TEMP DATA FROM EACH MONTH / YEAR %%%%%%%%%

% First make a vector of month names
months ={'01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12'};

% Loop through each of the files, reading them in and extracting the temp layer (each loop should take abo%ut 4 sec)
% Figure out the size of the TSA array
[nx,ny] = size(TSA);

% How many years are we using?
year_ct = 5;

% Make a large 3D array of zeros to accept the 60 months of TSA infomation 
% (use ';' to suppress the output of this statement)
TSA_out = zeros(year_ct*12,nx,ny);

% Zero the month counter
count = 0; 

for year =1:year_ct
    for month = 1:12
        %count up how many times we have done this. 
        count = count+1
        filename = char(strcat(dir_clm,'/','CLM2014Tutorial_5yr1850test.clm2.h0.000',char(num2str(year)),'-',char(months(month)),'.nc'))
        
        % read in file
        fileid = netcdf.open(filename,'NC_NOWRITE');

        % Choose a variable name
        varid = netcdf.inqVarID(fileid,'TSA');

        % Read in the data from this file and variable. It's data type is 'double precision' so we need to say that here...
        TSA = squeeze(netcdf.getVar(fileid,varid,'double'));
        
        % As with the single month, we need to rotate change the units of the data and remove the ocean values
	    TSA = TSA';     
        TSA(find(TSA>10^34)) = nan;   
        TSA = TSA - 273.13;
        
        % Put the modified TSA data into the large array. 
        TSA_out(count,:,:) = TSA; 
    end   
end


% Plot four months from the first year to make sure they look OK
figure
set(gcf,'name','Temperatures C: January to April')

% Make some labels for the different months 
labels = {'January','February','March','April'}

for t = 1:4
   %make a grid of 2x2 plots, within the main figure window
   subplot(2,2,t)
   pcolor(lons,lat,squeeze(TSA_out(t,:,:)))
   colorbar; shading flat
   title(labels(t))
end

%%%%%%%%%%%%%%%%%%%%%%%% CREATE ANNUAL TIME SERIES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Calculate the mean for the first year
mean_airtemp = mean(TSA_out(1:12,:,:),1);

% Calculate the mean for the all years
mean_airtemps = zeros(year_ct,nx,ny);
for y=1:year_ct
    start = (y-1) * 12+1;
    endy = start + 11;
    mean_airtemps(y,:,:) = mean(TSA_out(start:endy,:,:),1); 
end

% Look at mean temperatures for five years
figure
% Make labels for different years
labels = {'Year 01','Year 02','Year 03','Year 04','Year 05'}
set(gcf,'name','Annual Air Temperatures degrees C')
for y = 1:year_ct
   %make a grid of 2x3 plots, within the main figure window
   subplot(2,3,y)
   pcolor(lons,lats,squeeze(mean_airtemps(y,:,:)))
   colorbar;shading flat
   title(labels(y))
end

figure
set(gcf,'name','Temperature Difference Plot')
% look at a difference map between two years, just to confirm that they're not the same
pcolor(lons,lats,squeeze(mean_airtemps(1,:,:) - mean_airtemps(2,:,:)))
colorbar;shading flat


%%%%%%%%%%%%%% CALCULATE ANNUAL MEAN AND INTERANNUAL STANDARD DEVIATION OF MEANS %%%%%%%%%%%%%%%%%%

% get the number of years in your analysis (should be 5 here, but that may change for future stuff)

% Calculate the mean of the annual means
interannual_mean_airtemps = squeeze(mean(mean_airtemps,1));

% Calculate the standard deviation of the annual means
stdev_mean_airtemps = std(mean_airtemps,1);

figure
set(gcf,'name','Mean and StDev')
subplot(2,1,1)
pcolor(lons,lats,interannual_mean_airtemps)
% Give the plot a title, so you remember what it is...
title('Mean Annual Temperature')
shading flat
colorbar

subplot(2,1,2)
pcolor(lons,lats,squeeze(stdev_mean_airtemps))
title('Standard Deviation Annual Temperature')
shading flat
colorbar


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% NOW EXTRACT A REGION OF THE DATA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% We're going to look at Australia!
% define the extent of the area you're interested in
%australia  = interannual_mean_airtemps(110:115,35:80)    %(110,155,-45,10)

[a lat_min] = min(abs(lat--45))
[a lat_max] = min(abs(lat-10))
[a lon_min] = min(abs(lon-110))
[a lon_max] = min(abs(lon-155))


figure
set(gcf,'name','Australia Temperatures')
pcolor(interannual_mean_airtemps(lat_min:lat_max,lon_min:lon_max))
shading flat
colorbar


%%%%%%%%%%%%%%%%%%%%% MAKE  MOVIE OF ANNUAL TEMPERATURE %%%%%%%%%%%%%%%%%%%%%

figure
set(gcf,'name','movie monthly_temperature')
for m = 1:year_ct*12
   pcolor(lons,lats,squeeze(TSA_out(m,:,:)))
   %make the colormap (Clim) the same for each month, so that they can be looked at in sequence
   set(gca,'Clim',[-20 40]);
   colorbar
   title('Monthly Annual Temperature')
   shading flat
  
   F(m) = getframe
   %it is important to not click on other things on the screen while the movie is begin generated!
end

%Play the movie
figure
movie(F)

%Save the movie as an avi file
movie2avi(F,'Global_temp_movie.avi','compression','None')

%%%%%%%%%%%%%%% What about a slightly more boring plot? %%%%%%%%%%%%%

% Get monthly temperature for coordinates lat/lon
[a lat_point] = min(abs(lat-35))
[a lon_point] = min(abs(lon-255))

temp_point = TSA_out(:,lat_point,lon_point);

figure
set(gcf,'name','Boulder Monthly Temp')
plot(temp_point,'-ok')
% '-ok' tells matlab what points to use. 
% In this case 'o' is a circle, '-' is a line joining circles and
% 'k' is the colour blacK. 
xlabel('Months');ylabel('Temperature C')














